import java.util.*;

/**
 * Class that represents a bingo card.  A bingo card
 * has columns for B, I, N, G and O.  Each column has
 * 5 rows.  Each space can be filled or not.
 */
public class BingoCard
{
  ///////// fields ////////////////
  /** tracks which squares are filled */
  private boolean[][] filledArray = new boolean[5][5];
  /** tracks the balls on the card at each pos */
  private BingoBall[][] ballArray = new BingoBall[5][5];
  /** Random number generator */
  private Random randNumGen = new Random();
  
  ///////// constructor ///////////
  
  /**
   * No argument constructor the filled
   * array will be all false.  This fills
   * the bingo ball array (picks the balls
   * for each square)
   */
  public BingoCard()
  {
    fillBallArray();
  }
  
  /**
   * Method to pick the ball values for
   * each square on the card
   */
  private void fillBallArray()
  {
    List[] listArray = new List[5];
    int index = 0;
    
    // get the balls to pick from
    for (int col = 0; col < 5; col++)
    {
      listArray[col] = new ArrayList();
      for (int num = 1; num <  15; num++)
      {
        listArray[col].add(new BingoBall(col * 15 + num,
                                         col));
      }
    }
    
    // Now loop picking the 5 balls in each col
    for (int col = 0; col < 5; col++)
    {
      for (int row = 0; row < 5; row++)
      {
        index = randNumGen.nextInt(listArray[col].size());
        this.ballArray[row][col] = (BingoBall)
          listArray[col].remove(index);
      }
    }
  }
  
  /**
   * Method to check for a win (5 filled
   * horizontal, vertical, or diagonal)
   */
  public boolean checkForWin()
  {
    
    // check horizontal rows
    for (int i = 0; i < 5; i++)
    {
      if (filledArray[i][0] == true &&
          filledArray[i][1] == true &&
          filledArray[i][2] == true &&
          filledArray[i][3] == true &&
          filledArray[i][4] == true)
      {
        System.out.println();
        System.out.println("Horizontal Win on row " + i);
        return true;
      }
    }
    
    // check vertical columns
    for (int i = 0; i < 5; i++)
    {
      if (filledArray[0][i] == true &&
          filledArray[1][i] == true &&
          filledArray[2][i] == true &&
          filledArray[3][i] == true &&
          filledArray[4][i] == true)
      {
        System.out.println();
        System.out.println("Vertical Win on column " + i);
        return true;
      }
    }
    
    /* check diagonal top left to bottom
     * right */
    if (filledArray[0][0] == true &&
        filledArray[1][1] == true &&
        filledArray[2][2] == true &&
        filledArray[3][3] == true &&
        filledArray[4][4] == true)
    {
      System.out.println();
      System.out.println("Left to right diagonal win");
      return true;
    }
    
    /* check diagonal top right to bottom
     left */
    if (filledArray[0][4] == true &&
        filledArray[1][3] == true &&
        filledArray[2][2] == true &&
        filledArray[3][1] == true &&
        filledArray[4][0] == true)
    {
      System.out.println();
      System.out.println("Right to left diagonal win");
      return true;
    }
    else return false;
  }
  
  /**
   * Method that will search for a ball in a column
   * @param ball the ball to search for
   * @return int the row of the ball if found
   * else -1 if not found
   */
  private int searchColumnForBall(BingoBall ball)
  {
    int row = -1;
    int col = ball.getLetterNum();
    for (int i = 0; i < 5; i++)
    {
      if (this.ballArray[i][col].compareTo(ball) 
            == 0)
      {
        filledArray[i][col]  = true;
        row = i;
      }
    }
    return row;
  }
  
  /**
   * Method to changed the filled array
   * based on the passed bingo ball
   * @param ball the current picked ball
   * @return true when win else false
   */
  public boolean addBall(BingoBall ball)
  {
    searchColumnForBall(ball);
    return checkForWin();
  }
  
  /**
   * Method to output information about this card
   * @return a string that shows info on the card
   */
  public String toString()
  {
    BingoBall ball = null;
    String result = "";
    for (int row = 0; row < 5; row++)
    {
      for (int col = 0; col < 5; col++)
      {
        ball = ballArray[row][col];
        result = result + ball.toString() + ":" +
          filledArray[row][col] + " ";
      }
      result = result + "\n";
    }
    return result;
  }
      

  public static void main (String[] args)
  {
    BingoCard card = new BingoCard();
    System.out.println(card);
  }
}