/*
* File: Breakout.java
* -------------------
* This file will eventually implement the game of Breakout.
*/
import acm.graphics.*;
import acm.program.*;
import acm.util.*;
import java.applet.*;
import java.awt.*;
import java.awt.event.*;
public class Breakout extends GraphicsProgram
{
  /** Width of the game display (all coordinates are in pixels) */
  private static final int WIDTH = 400;
  /** Height of the game display */
  private static final int HEIGHT = 600;
  
  /** Width of the paddle */
  private static final int PADDLE_WIDTH = 60;
  /** Height of the paddle */
  private static final int PADDLE_HEIGHT = 10;
  /** Offset of the paddle up from the bottom */
  private static final int PADDLE_Y_OFFSET = 30;
  /** Number of bricks per row */
  private static final int NBRICKS_PER_ROW = 10;
  /** Number of rows of bricks */
  private static final int NBRICK_ROWS = 10;
  /** Separation between bricks */
  private static final int BRICK_SEP = 4;
  /** Width of a brick */
  private static final int BRICK_WIDTH =
    WIDTH / NBRICKS_PER_ROW - BRICK_SEP;
  /** Height of a brick */
  private static final int BRICK_HEIGHT = 8;
  /** Radius of the ball in pixels */
  private static final int BALL_RADIUS = 10;
  /** Width of the ball in pixels */
  private static final int BALL_WIDTH = BALL_RADIUS * 2;
  /** Offset of the top brick row from the top */
  private static final int BRICK_Y_OFFSET = 70;
  /** Number of turns */
  private static final int NTURNS = 3;
  
  private GRect paddle = null;
  private GPoint last = null;
  private GObject gObj = null;
  private double velX;
  private double velY = 3;
  private RandomGenerator rGen = 
    new RandomGenerator();
  private GOval ball = null;
  private int turn = 0;
  private int sleepTime = 25;
  private int numBricksLeft = NBRICK_ROWS * NBRICKS_PER_ROW;
  
  /* Static method: main(args) */
  /**
   * Runs the program as an application. This method differs
   * from the simplest possible boilerplate in that it passes
   * parameters to specify the dimensions of the court.
   */
  public static void main(String[] args) {
    String[] sizeArgs = { "width=" + WIDTH, 
      "height=" + HEIGHT };
    new Breakout().start(sizeArgs);
  }
  
  public void mousePressed(MouseEvent e) {
    last = new GPoint(e.getPoint());
    gObj = getElementAt(last);
  }
  
  public void mouseDragged(MouseEvent e) {
    if (gObj != null && gObj == paddle) {
       movePaddle(e);
    }
  }
  
  public void mouseClicked(MouseEvent e) {
    paddle.sendToFront();
  }
  
  public void movePaddle(MouseEvent e) {
    GPoint pt = new GPoint(e.getPoint());
    double xDiff = pt.getX() - last.getX();
    if (paddle.getX() + xDiff < 0)
      xDiff = 0;
    else if (paddle.getX() + xDiff > 
             (WIDTH - PADDLE_WIDTH))
      xDiff = 0;
    paddle.move(xDiff, 0);
    last = pt;
  }
  
  private void setUpPaddle()
  {
       paddle = new GRect((WIDTH / 2) - (PADDLE_WIDTH / 2),
                       HEIGHT - PADDLE_Y_OFFSET - 
                       PADDLE_HEIGHT,
                       PADDLE_WIDTH,
                       PADDLE_HEIGHT);
    paddle.setColor(Color.BLACK);
    paddle.setFilled(true);
    this.add(paddle);
    addMouseListeners();
    System.out.println("paddle x " + paddle.getX() +
                       " paddle y " + paddle.getY());
  }
  
  private void setUpBlocks()
  {
  
  GRect rect = null;
    Color[] colorArray = {Color.RED,
      Color.ORANGE, Color.YELLOW, Color.GREEN,
      Color.CYAN};
    
    int xStart = (WIDTH - ((NBRICKS_PER_ROW * 
                          BRICK_WIDTH) +
      ((NBRICKS_PER_ROW - 1) * BRICK_SEP))) / 2;
  
    int yPos = BRICK_Y_OFFSET;
    for (int i = 0; i < colorArray.length; i++) {
      Color currColor = colorArray[i];
      for (int j = 0; j < 2; j++) {
        int xPos = xStart;
        for (int x = 0; x < NBRICKS_PER_ROW; x++) {
          rect = new GRect(xPos,yPos,
                           BRICK_WIDTH,
                           BRICK_HEIGHT);
        rect.setColor(currColor);
        rect.setFilled(true);
        this.add(rect);
        xPos = xPos + BRICK_WIDTH + BRICK_SEP;
        }
        yPos = yPos + BRICK_HEIGHT + BRICK_SEP;
      }
    }
  }
  
  public void setUp()
  {
    // set up the blocks
    setUpBlocks();
    
    // create the paddle
    setUpPaddle();
    
  }
  
  private void createBall()
  {
    ball = new GOval((WIDTH / 2) - BALL_RADIUS,
      BRICK_Y_OFFSET + (NBRICK_ROWS * 
                        (BRICK_HEIGHT + BRICK_SEP)),
                           BALL_WIDTH,BALL_WIDTH);
    ball.setColor(Color.BLACK);
    ball.setFilled(true);
    this.add(ball);
    velX = rGen.nextDouble(1.0, 3.0);
    if (rGen.nextBoolean(0.5)) velX = -velX;
  }
  
  private GObject getCollidingObject()
  {
    GObject collider = null;
    double x = ball.getX();
    double y = ball.getY();
    
    // try top left corner
    collider = getElementAt(x,y);
    if (collider != null)
      return collider;
    
    // try top right corner
    collider = getElementAt(x + BALL_WIDTH,y);
    if (collider != null)
      return collider;
    
    // try bottom left corner
    collider = getElementAt(x,BALL_WIDTH + y);
    if (collider != null)
      return collider;
    
    // try bottom right corner
    collider = getElementAt(x + BALL_WIDTH, y + BALL_WIDTH);
    return collider;
  }
  
  public void startGame() throws Exception
  {
    // reset number of turns taken
    turn = 0;
    
    // create the first ball
    createBall();
    
    // while not finished turns
    while (turn < NTURNS)
    {
      // move the ball
      ball.move(velX,velY);
      
      // check if collided with paddle or block
      GObject collider = getCollidingObject();
      if (collider != null)
      {
        velY = -velY;
        if(collider != paddle)
        {
          remove(collider);
          numBricksLeft--;
          if (numBricksLeft == 0)
          {
            GLabel message = new GLabel("You won!");
            message.setFont(new Font("Serif", Font.BOLD, 18));
            message.setLocation((WIDTH/2) - (message.getWidth()/2),
                               BRICK_Y_OFFSET);
            this.add(message);
          }
        }
      }
      
      // check if hit walls
      if (ball.getX() < 0) 
        velX = -velX;
      else if (ball.getX() + BALL_WIDTH > WIDTH)
        velX = -velX;
      else if (ball.getY() < 0)
        velY = -velY;
      else if (ball.getY() > HEIGHT)
      {
        turn++;
        if (turn < 3)
           createBall();
        else {
          GLabel message = new GLabel("You lost!");
          message.setFont(new Font("Serif", Font.BOLD, 18));
          message.setLocation((WIDTH/2) - (message.getWidth() /2),
                                      HEIGHT - 5);
          add(message);
        }
      }
      Thread.sleep(sleepTime);
    }
  }
  
  /* Method: run() */
  /**
   * Runs the Breakout program.
   */
  public void run() {
    /* You fill this in, along with any subsidiary methods */
    setUp();
    try {
      startGame(); 
    } catch (Exception ex) {
    }
  }
}