 /*
 * BlackJack.java
 *
 * Created on April 11, 2006, 8:59 PM
 *
 * To change this template, choose Tools | Options and locate the template under
 * the Source Creation and Management node. Right-click the template and choose
 * Open. You can then make changes to the template in the Source Editor.
 */

/**
 * Logic class for BlackJack card game.
 * @author Jheinka
 */
public class BlackJack {
    
    /**
     * Reference to the BlackJack view.
     */
    private BlackJackGUI gui;    
    /**
     * User of the program that is currently playing the card game.
     */
    private Player humanPlayer = null;
    /**
     * Artificially simulated dealer to play against the human player.
     */
    private Player dealer = null;
    /**
     * Deck from which the game will be drawn from.
     */
    private CardPile dealerDeck = null;
    /**
     * The amount being bet on the current round.
     */
    private int currentBet = 0;
    
    /**
     * Returns the human player of the game.
     * @return human player of the blackjack game.
     */
    public Player getHumanPlayer() {
        return humanPlayer;
    }

    /**
     * Returns the computer player of the game.
     * @return computer player of the blackjack game.
     */
    public Player getDealer() {
        return dealer;
    }
    
    
    /**
     * Creates a new instance of BlackJack.
     * @param g GUI for displaying the game.
     * @param playerName name of the human player that is going to be playing the
     *                   card game.
     */
    public BlackJack(String playerName, BlackJackGUI g) {
        gui = g;
        humanPlayer = new Player(playerName);
        humanPlayer.setScore(100);
        dealer = new Player("Dealer");
        createDeck();
        dealerDeck.shuffle();
    }
    
    /**
      * Sets up six decks for the game as a <code>CardPile</code>.
      */
     private void createDeck() {
         dealerDeck = new CardPile();

         for (int i = 0; i < 6; i++) {
         	Deck newDeck = new Deck();
         	newDeck.fillDeck();
             dealerDeck.addCards(newDeck);
         }
     }
     
     /**
      * Distributes cards appropriately to the players.
      * @param betAmount The amount of money being wagered on the upcoming round.
      */
     public void dealCards(int betAmount) {
     	//System.out.println("IN dealCards BlackJack");
        Card dealtCard = null; 
        currentBet = betAmount;        
        humanPlayer.setScore(humanPlayer.getScore() - betAmount);
        
        //Refill if deck is nearing empty
        if (dealerDeck.size() <= 15) {
            createDeck();
            dealerDeck.shuffle();
        }
         
        //Deal first card to player
        dealtCard = dealerDeck.removeTopCard();
        dealtCard.setFaceUp(true);
        humanPlayer.getHand().addCard(dealtCard);
         
        //Deal first card to dealer
        dealtCard = dealerDeck.removeTopCard();
        dealtCard.setFaceUp(false);         //first dealer card is face down!
        dealer.getHand().addCard(dealtCard);
        
        //Deal second card to player
        dealtCard = dealerDeck.removeTopCard();
        dealtCard.setFaceUp(true);
        humanPlayer.getHand().addCard(dealtCard);
         
        //Deal second card to dealer
        dealtCard = dealerDeck.removeTopCard();
        dealtCard.setFaceUp(true);
        dealer.getHand().addCard(dealtCard);
         
        //System.out.println("About to call deal Cards in GUI");
        //System.out.println("Dealer HAND Size after Deal: " + dealer.getHand().size());
        //gui.dealCards(humanPlayer, dealer);
        //gui.dealPlayCards(humanPlayer);
        gui.setDealerHand(dealer.getHand());
        gui.showDealedCards(humanPlayer, dealer);
     }
     
     /**
      * Allow player to draw another card to add to his hand.
      */
     public void playerHit() {
    	 //System.out.println("Dealer Hand Size 0: " + dealer.getHand().size());
         Card dealtCard = dealerDeck.removeTopCard();
         dealtCard.setFaceUp(true);
         humanPlayer.getHand().addCard(dealtCard);
         
         if (humanPlayer.getHand().calculateValue() > 21) {
        	 gui.playerBust();
        	 gui.inRound = false;
             playerLoseBet();
         }
         //System.out.println("DealerHand size 1: " + dealer.getHand().size());
         gui.playerHit(dealtCard, humanPlayer.getHand());
         //System.out.println("Dealer Hand size 2: " + dealer.getHand().size());
     }
     
     /**
      * Player is has finished drawing cards for this round.
      */
     public void playerStand() {
    	 //System.out.println("Dealer Hand size: " + dealer.getHand().size());
         drawForDealerTurn();
         //determineWinner();
     }
     
     /**
      * Automatically draws cards to dealer hand until the total value of the dealer's hand exceeds 16.
      */
     private void drawForDealerTurn() {
         //Show dealer first card.
    	 if (dealer.getHand().size() == 0) {
    		 //System.out.println("Dealer hand 0");
    	 }
         dealer.getHand().getCardAt(1).setFaceUp(true);
         gui.setDealerCardsFaceUp(dealer.getHand());
         
         //Dealer must draw if he does not beat the player until he has more than 16
         while (dealer.getHand().calculateValue() <= 16) {
        	 //System.out.println("Dealer Hand Value: " + dealer.getHand().calculateValue() );
             Card dealtCard = dealerDeck.removeTopCard();
             dealtCard.setFaceUp(true);
             dealer.getHand().addCard(dealtCard);
             //System.out.println("About to call add Card Dealer");
             //System.out.println(gui.getImage(dealtCard));
             //gui.dealerHit(dealtCard, dealer.getHand());
             //if (dealer.getHand().calculateValue() > 21) {
            //	 //System.out.println("About to BREAK");
              //   playerWinBet();
            	// break;
             //}

             
         }
         //gui.setDealer(dealer);
         gui.allDealerHits(dealer.getHand());
         //gui.dealerCardAnimation();
     }
     
     /**
      * Compares the hands of the dealer and player and decides who has won.
      */
     public void determineWinner() { 
    	 //System.out.println("IN determine Winner");
         int playerTotal = humanPlayer.getHand().calculateValue();
         int dealerTotal = dealer.getHand().calculateValue();
         
         if (playerTotal > 21) {
        	 gui.playerBust();
        	 gui.inRound = false;
             playerLoseBet();
         } else if (dealerTotal > 21) {
        	 gui.dealerBust();
        	 gui.inRound = false;
             playerWinBet();
         } else if (playerTotal <= dealerTotal) {
        	 gui.dealerWinner();
        	 gui.inRound = false;
             playerLoseBet();
         } else {
        	 gui.playerWinner();
        	 gui.inRound = false;
             playerWinBet();
         }
     }
     
     
     /**
      * Player loses the money amount that was bet for the round.
      */
     private void playerLoseBet() {
         currentBet = 0;
         //gui.show winner
         emptyCardHands();
     }
          
     /**
      * Player wins double the amount of money that he bet on current round.
      */
     private void playerWinBet() {
         humanPlayer.setScore(humanPlayer.getScore() + (2 * currentBet));
         gui.updateBalance(humanPlayer.getScore());
         gui.inRound = false;
         //gui.show winner
         currentBet = 0;
         emptyCardHands();
     }
     
     /**
      * Clear all cards from dealer and player hands.
      */
     private void emptyCardHands() {
    	 //System.out.println( "Emptying Cards");
         while (humanPlayer.getHand().size() > 0) {
             humanPlayer.getHand().removeTopCard();
         }
         while (dealer.getHand().size() > 0) {
             dealer.getHand().removeTopCard();
         }
         //The piles that are being added are empty
         //gui.hideCards();
     }
}
