  

/**
 * Card.java
 *
 * <code>Card</code> represents any card used in the game.
 *
 * @author <a href="mailto:kaika@cc.gatech.edu">Veronica Nicole Peshterianu</a>
 * @version 1.0
 */
public class Card {


    /**
     * String value that holds the suit of the card
     */
    private String suit;
    //the suit of the card




    /**
     * int value that holds the rank of the card
     */
    private Rank rank;
    //the rank of the card
    




    /**
     * boolean value that holds whether the card is face up or face down
     */
    private boolean faceUp;
    //whether the card is face up or face down






    
    /**
     *  <code>getSuit</code> returns the suit of the card
     *
     * @return a <code>String</code> value of the suit
     */
    public String getSuit(){
        return suit;
    }// end getSuit()


    /**
     * <code>setSuit</code> sets the suit of the given card
     *
     * @param cardSuit a <code>String</code> value of the suit
     */
    public void setSuit(String cardSuit){
        this.suit = cardSuit;
    }// end setSuit()


    
    /**
     * <code>getRank</code> gets the rank of the given card
     *
     * @return an <code>int</code> value of the rank
     */
    public Rank getRank(){
        return rank;
    }// end getRank()


    /**
     *  <code>setRank</code> sets the rank of the given card
     *
     * @param cardRank an <code>int</code> value of the rank
     */
    public void setRank(Rank cardRank){
        this.rank = cardRank;
    } // end setRank()



    /**
     * <code>getFaceUp</code> tells the user whether or not the card is face up
     *
     * @return a <code>boolean</code> value, true if the card is face up.
     */
    public boolean getFaceUp(){
        return faceUp;
    }// end getFaceUp()


    /**
     * <code>setFaceUp</code> sets whether the card is face up or face down
     *
     * @param isFaceUp a <code>boolean</code> value
     */
    public void setFaceUp(boolean isFaceUp){
        this.faceUp = isFaceUp;
    } // end setFaceUp()


	

    /**
     * Creates a new <code>Card</code> instance.
     *
     * @param cardRank an <code>int</code> value containing the rank of the card
     * @param cardSuit a <code>String</code> value containing the suit of the card
     */
    public Card(Rank cardRank, String cardSuit){
        
        //initializes a new Card with the given rank and suit
        this.setRank(cardRank);
        this.setSuit(cardSuit);

    } // end of Card(int,String)


    /**
     * Converts the rank and suit into a string in the format "[Rank] of 
     * [Suit]".  This presents us with a useful way of printing out the contents 
     * of a <code>CardPile</code> in an easily readable format or to perform 
     * other similar functions.
     *
     * @return a <code>String</code> containing the names of the rank and suit of
     *         the card
     */
    public String toString(){

      int curRank = ((Integer) this.getRank().getValues().get(0)).intValue();
      // keeps the rank of the current card

      String curSuit = this.getSuit();
      // keeps the suit of the current card

      String rankAndSuit = null;
      // the String that is returned containing the current rank and suit

      switch(curRank) {

        // switch statement allows for the method to quickly evaluate the rank
        // and choose the appropriate statement return as the "rank and suit"
        // helps avoid the redundant if-then-else statement

      case 2:
      case 3:
      case 4:
      case 5:
      case 6:
      case 7:
      case 8:
      case 9:
      case 10:
        rankAndSuit = curRank + " of " + curSuit; 
        break;

        /* because cards of ran 2-10 do not have special names, their rankAndSuit
           value will simply be the integer value of the rank and the name of the
           suit */

      case 11: rankAndSuit = "Jack of " + curSuit; 
        break;
      case 12: rankAndSuit = "Queen of " + curSuit;
        break;
      case 13: rankAndSuit = "King of " + curSuit;
        break;
      case 14: rankAndSuit = "Ace of " + curSuit; 
        break;

        /* because the cards of rank 11-14 are face cards and have names, these 
           cases will return the actual name of the card */

      case 15: rankAndSuit = "Joker"; 
        break;

        // because Joker is wild, it has the highest rank.

      default: System.out.println("No such rank!"); 
        break;

        /* this is a safety net in case your rank turns out to be a negative 
           number, 1, 0, or higher than 15. */

      } // end switch(curRank)

      return rankAndSuit;

    } // end toString()

} // Card


        

