 import java.util.ArrayList;
import java.util.Random;


/**
 * The CardPile class represents a collection of <code>Card</code>s.  It allows
 * <code>Card</code>s to be grouped together, and it can be shuffled to
 * randomize the order of the <code>Card</code>s.  <code>Card</code>s can be
 * drawn from the top of the pile or the bottom of the pile, or a
 * <code>Card</code> can be picked from the middle by specifying an index.
 */
public class CardPile {

    /**
     * The <code>Card</code>s contained in the pile.
     */
    private ArrayList /*<Card>*/ cards;


    /**
     * Creates an empty <code>CardPile</code>.
     */
    public CardPile() {
        cards = new ArrayList/*<Card>*/();
 
    }// end CardPile()


    /**
     * Returns the size of the pile.
     *
     * @return the size of the pile
     */
    public int size() {
        return cards.size();
 
    }// end size()


    /**
     * Randomizes the order of the <code>Card</code>s in the pile.
     */
    public void shuffle() {

        int size = cards.size();

        /* if there are less than two cards there is nothing to shuffle! */
        if (size < 2) {
            return;
        } // end if

        Random random = new Random();

        int shuffleCount = 0;  // keep track of how many times we've shuffled

        while (shuffleCount < (size / 2)) {
            int a = random.nextInt(size);
            int b = random.nextInt(size);
            Card temp = (Card) cards.get(a);
            cards.set(a, cards.get(b));
            cards.set(b, temp);
            shuffleCount++;
        }// end while
 
    }// end shuffle()


    /**
     * Returns the top <code>Card</code> in the pile.
     * The top card is the <code>Card</code> that was added last.
     *
     * @return the top card
     */
    public Card getTopCard() {
        return (Card) cards.get(0);
 
    }// end getTopCard()
    

    /**
     * Removes the top <code>Card</code> from the pile.
     * The top card is the <code>Card</code> that was added last.
     *
     * @return the top card
     */
    public Card removeTopCard() {
        return (Card) cards.remove(0);
    
    }// end removeTopCard()



    /**
     * Returns the bottom <code>Card</code> in the pile.
     * The bottom <code>Card</code> is the <code>Card</code> that was added 
     * first.
     *
     * @return the bottom card
     */
    public Card getBottomCard() {
        return (Card) cards.get(cards.size() - 1);
    
    }// end getBottomCard()


    /**
     * Removes the bottom <code>Card</code> from the pile.
     * The bottom <code>Card</code> is the <code>Card</code> that was added 
     * first.
     *
     * @return the bottom card
     */
    public Card removeBottomCard() {
        return (Card) cards.remove(cards.size() - 1);
    
    }// end removeBottomCard()


    /**
     * Returns the <code>Card</code> at the specified index in the pile.
     *
     * @param i the index of the card to be returned
     * @return the card at the specified index
     */
    public Card getCardAt(int i) {
        return (Card) cards.get(i);
 
    }// end getCardAt()


 /**
     * Removes the <code>Card</code> at the specified index from the pile.
     *
     * @param i the index of the card to be returned
     * @return the card at the specified index
     */
    public Card removeCardAt(int i) {
        return (Card) cards.remove(i);
 
    }// end removeCardAt()

    
 /**
     * Adds a <code>Card</code> to the top of the pile.
     *
     * @param card the card to be added to the pile
     */
    public void addCard(Card card) {
        cards.add(0, card);
 
    } // end addCard()


 /**
     * Adds a set of <code>Cards</code> to the top of the pile.
     *
     * @param newCards the card to be added to the pile
     */
    public void addCards(CardPile newCards) {
     int i;
     int newSize = newCards.size(); 
     //gets the size of the set of cards to add
   
     for (i = 0; i < newSize; i++){
      cards.add(0, newCards.getCardAt(i));
     }// end for

     //loops through the list, adding the new cards one by one
     //to the top of the stack

    }// end addCards()

 


    
}
