 /**
 * The <code>Player</code> class represents either a user or computer
 * controlled player in Blackjack.  <code>Player</code> maintains his hand,
 * his current bet, his current score, and whether or not he wants to hit
 * or stand on the next play.
 *
 * @author <a href="mailto:">Veronica Nicole Peshterianu</a>
 * @version 1.0
 */
public class Player {


  
  /*
   * The player's hand.
   */
  private Hand hand;
  

  
  /**
   * The player's current score.
   */
  private int score;


  /*
   * The player's bet.
   */
  private int bet;


  /**
   * The player's name.
   */
  private String name;
  
  
  /*
   * Whether the player wants to hit or stand
   */
  private boolean hasBlackjack;

  

  
  /**
   * Returns the player's score.
   *
   * @return the player's score
   */
  public int getScore() {
    return score;

  } // end getScore()


  /**
   * Sets the player's score to the given value.
   *
   * @param newScore the player's new score
   */
  public void setScore(int newScore) {
    this.score = newScore;

  } // end setScore()


  

  /**
   * Returns the player's bet.
   *
   * @return the player's bet.
   */
  public int getBet() {
	 return bet;

	 
  }// end getBet()

    
  

  /**
   * Returns the player's name.
   *
   * @return the player's name
   */
  public String getName() {
    return name;

  } // end getName()

  

 
  
  /**
   * Returns whether the player has a blackjack
   *
   * @return if the player has blackjack
   */
  public boolean getHasBlackjack() {
	 return hasBlackjack;

	 
  }// end hasBlackjack()

  
   /**
   * Sets whether the player has blackjack
   *
   * @param newBlackjack whether the player has blackjack
   */
  public void setHasBlackjack(boolean newBlackjack) {
	  this.hasBlackjack = newBlackjack;
	  
  }// end setHasBlackjack()
  
  
  /**
   * Returns the players hand
   * 
   * @return Hand
   */
  public Hand getHand() {
  	return hand;
  }



  /**
   * <code>placeBet</code> method allows the player to place
   * a <code>bet</code> from his current score
   * @param  newBet the new bet that needs to be set
   * @return new bet that the player has set
   */
  
	
  public int placeBet(int newBet){
	  this.bet = newBet;
	  return bet;	
	
  }// end placeBet



  
/**
 * <code>hitOrStand</code> method allows the computer to decide
 * whether he wants to hit or stand on the next play.
 * The dealer and the computer players stand on 17, therefore they
 * must keep drawing until their hand value is greater than 17.
 * After that, they stand regardless of what the human player has.
 * This method is only used by the computer player and the dealer.
 * 
 * @return boolean value of whether the player is standing
 */

  public boolean hitOrStand(){
	  if (hand.calculateValue() >= 17)
		  return false;
	  else
		  return true;

	  
  }// end hitOrStand




  

  /**
   * Creates a new <code>Player</code> with the given name.
   *
   * @param playerName the player's name
   */
  public Player(String playerName) {

    name = playerName;
    hand = new Hand(); 

  } // Player constructor
    
} // Player
