/**
 * The <code>Card</code> class represents a playing card for use in any typical
 * card game.  It has two attributes: a rank and a suit, which identify the
 * card.  It can be either face up or face down.
 *
 * @author <a href="mailto:kaika@cc.gatech.edu">Veronica Nicole Peshterianu</a>
 * @version 1.0
 */
public class Card {


  /**
   * The suit of the card.
   */
  private String suit;


  /**
   * The rank of the card.
   */
  private int rank;


  /**
   * Whether the card is face up or face down.
   */
  private boolean faceUp;


  /**
   * Returns the suit of the card.
   *
   * @return the suit of the card
   */
  public String getSuit(){
    return suit;
  }// end getSuit()


  /**
   * Sets the suit of the card to the given value.
   *
   * @param newSuit the new suit of the card
   */
  public void setSuit(String newSuit) {
    suit = newSuit;
  }


  /**
   * Returns the rank of the card.
   *
   * @return the rank of the card
   */
  public int getRank(){
    return rank;
  }// end getRank()


  /**
   * Sets the rank of the card to the given value.
   *
   * @param newRank the new rank of the card
   */
  public void setRank(int newRank) {
    rank = newRank;
  }


  /**
   * Returns whether or not the card is face up.
   *
   * @return true if the card is face up, false if the card is face down
   */
  public boolean getFaceUp(){
    return faceUp;
  }// end getFaceUp()


  /**
   * Sets whether the card is face up or face down
   *
   * @param isFaceUp <code>boolean</code> value, true if the card should be
   *                 face up and false if the card should be face down
   */
  public void setFaceUp(boolean isFaceUp){
    this.faceUp = isFaceUp;
  } // end setFaceUp()


  /**
   * Compares the rank of this card to the rank of the given card and returns 
   * whether or not this card's rank is higher than the given card's.
   *
   * @param givenCard the card that this card is being compared to
   * @return true if this card has a greater rank, false if the other card has 
   *         a greater rank
   */
  public boolean isGreaterThan(Card givenCard) {

    if(this.getRank() > givenCard.getRank())
      return true;
    else
      return false;

  } // end isGreaterThan


  /**
   * Converts the rank and suit into a string in the format "[Rank] of 
   * [Suit]".  This presents us with a useful way of printing out the contents 
   * of a <code>CardPile</code> in an easily readable format or to perform 
   * other similar functions.
   *
   * @return a <code>String</code> containing the names of the rank and suit of
   *         the card
   */
  public String toString(){

    int curRank = this.getRank();
    // keeps the rank of the current card

    String curSuit = this.getSuit();
    // keeps the suit of the current card

    String rankAndSuit = null;
    // the String that is returned containing the current rank and suit

    switch(curRank) {

      // switch statement allows for the method to quickly evaluate the rank
      // and choose the appropriate statement return as the "rank and suit"
      // helps avoid the redundant if-then-else statement

    case 2:
    case 3:
    case 4:
    case 5:
    case 6:
    case 7:
    case 8:
    case 9:
    case 10:
      rankAndSuit = curRank + " of " + curSuit; 
      break;

      /* because cards of ran 2-10 do not have special names, their rankAndSuit
         value will simply be the integer value of the rank and the name of the
         suit */

    case 11: rankAndSuit = "Jack of " + curSuit; 
      break;
    case 12: rankAndSuit = "Queen of " + curSuit;
      break;
    case 13: rankAndSuit = "King of " + curSuit;
      break;
    case 14: rankAndSuit = "Ace of " + curSuit; 
      break;

      /* because the cards of rank 11-14 are face cards and have names, these 
         cases will return the actual name of the card */

    case 15: rankAndSuit = "Joker"; 
      break;

      // because Joker is wild, it has the highest rank.

    default: System.out.println("No such rank!"); 
      break;

      /* this is a safety net in case your rank turns out to be a negative 
         number, 1, 0, or higher than 15. */

    } // end switch(curRank)

    return rankAndSuit;

  } // end toString()


  /**
   * Creates a new <code>Card</code>, given a rank and a suit.
   *
   * @param cardRank an <code>int</code> representing the rank of the 
   *                 card
   * @param cardSuit a <code>String</code> representing the suit of the
   *                 card
   */
  public Card(int cardRank, String cardSuit){

    //initializes a new Card with the given rank and suit
    this.setRank(cardRank);
    this.setSuit(cardSuit);

  } // end of Card(int,String)


} // end of class Card


        

