import java.util.ArrayList;
import java.util.Random;


/**
 * The <code>CardPile</code> class represents a collection of <code>Card</code>
 * objects.  It allows cards to be grouped together, and it can be shuffled to 
 * randomize the order of the cards.  Cards can be drawn from the top of the 
 * pile or the bottom of the pile, or a card can be picked from the middle by 
 * specifying an index.
 */
public class CardPile {

  /**
   * The cards contained in the pile.
   */
  private ArrayList cards;


  /**
   * Creates an empty <code>CardPile</code>.
   */
  public CardPile() {
    cards = new ArrayList();
  }


  /**
   * Returns the size of the pile.
   *
   * @return the size of the pile
   */
  public int cardCount() {
    return cards.size();
  }


  /**
   * Randomizes the order of the cards in the pile.
   */
  public void shuffle() {

    int size = cards.size();
    /* if there are less than two cards there is nothing to shuffle! */
    if (size < 2) {
        return;
    }

    Random random = new Random();
    int shuffleCount = 0;  // keep track of how many times we've shuffled
    while (shuffleCount < (size / 2)) {
        int a = random.nextInt(size);
        int b = random.nextInt(size);
        Card temp = (Card) cards.get(a);
        cards.set(a, cards.get(b));
        cards.set(b, temp);
        shuffleCount++;
    }
  }


  /**
   * Returns the top card in the pile.
   * The top card is the card that was added last.
   *
   * @return the top card
   */
  public Card getTopCard() {
    return (Card) cards.get(0);
  }


  /**
   * Removes the top card from the pile and returns it.
   *
   * @return the top card
   */
  public Card removeTopCard() {
    return (Card) cards.remove(0);
  }


  /**
   * Returns the bottom card in the pile.
   * The bottom card is the card that was added first.
   *
   * @return the bottom card
   */
  public Card getBottomCard() {
    return (Card) cards.get(cards.size() - 1);
  }


  /**
   * Removes the bottom card from the pile and returns it.
   *
   * @return the bottom card
   */
  public Card removeBottomCard() {
    return (Card) cards.remove(cards.size() - 1);
  }


  /**
   * Returns the card at the specified index in the pile.
   *
   * @param i the index of the card to be returned
   * @return the card at the specified index
   */
  public Card getCardAt(int i) {
    return (Card) cards.get(i);
  }
  

  /**
   * Removes the card at the specified index in the pile and returns it.
   * 
   * @param i the index of the card to be returned
   * @return the card at the specified index
   */
  public Card removeCardAt(int i) {
    return (Card) cards.remove(i);
  }


  /**
   * Adds a card to the top of the pile.
   *
   * @param card the card to be added to the pile
   */
  public void addCard(Card card) {
    cards.add(0, card);
  }


} // end of class CardPile
