/**
 * The <code>Player</code> class represents either a user or computer
 * controlled player in War.  <code>Player</code> maintains all of the player's
 * various card piles, including the Won Pile and the Draw Pile.  It also has
 * a method for automatically drawing cards from the Draw Pile and placing them
 * in the player's Hand to replace cards that were played in the previous
 * round.
 *
 * @author <a href="mailto:">Veronica Nicole Peshterianu</a>
 * @version 1.0
 */
public class Player {


  /**
   * The player's current score.
   */
  private int score;


  /**
   * The player's name.
   */
  private String name;


  /**
   * Draw Pile for the Player to draw cards from.
   *
   */
  private CardPile drawPile;


  /**
   * Won Pile for the Player to store cards that were won in a round.
   */
  private CardPile wonPile;
   

  /**
   * Gamble Pile to hold the contested cards in a round.
   */
  private CardPile gamblePile;


  /**
   * Hand for the Player to select cards from to play in a round.
   */
  private CardPile hand;


  /**
   * Returns the player's score.
   *
   * @return the player's score
   */
  public int getScore() {
    return score;

  } // end getScore()


  /**
   * Sets the player's score to the given value.
   *
   * @param newScore the player's new score
   */
  public void setScore(int newScore) {
    this.score = newScore;

  } // end setScore()


  /**
   * Returns the player's name.
   *
   * @return the player's name
   */
  public String getName() {
    return name;

  } // end getName()


  /**
   * Returns the player's draw pile.
   *
   * @return the player's draw pile
   */
  public CardPile getDrawPile() {
    return drawPile;
  }


  /**
   * Returns the player's won pile.
   *
   * @return the player's won pile
   */
  public CardPile getWonPile() {
    return wonPile;
  }


  /**
   * Returns the player's gamble pile.
   *
   * @return the player's gamble pile
   */
  public CardPile getGamblePile() {
    return gamblePile;
  }


  /**
   * Returns the player's hand.
   *
   * @return the player's hand
   */
  public CardPile getHand() {
    return hand;
  }


  /**
   * Checks how many cards are missing from the player's hand, and replaces the
   * missing cards with cards from the Draw Pile.
   */
  public void refillHand(){
    int toFill = 0;
    //temp int to hold how many cards need to be filled

    toFill = (5-hand.cardCount());
	
    while(toFill > 0){

      hand.addCard(drawPile.removeTopCard());

      /* gets the top card from the draw pile and immediately adds it to the 
         hand - this can also be done with a temp card to hold the top card */

      toFill--;
    }

  } // end refillHand


  public void refillDrawPile() {
      wonPile.shuffle();
      int cardCount = wonPile.cardCount();
      for (int i = 0; i < cardCount; i++) {
          Card temp = wonPile.removeTopCard();
          drawPile.addCard(temp);
      }
  }


  /**
   * Creates a new <code>Player</code> with the given name.
   *
   * @param playerName the player's name
   */
  public Player(String playerName) {

    name = playerName;
    drawPile = new CardPile();
    wonPile = new CardPile();
    gamblePile = new CardPile();
    hand = new CardPile();

  } // Player constructor
    
} // Player
