import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.net.URL;

import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JTextArea;
import javax.swing.KeyStroke;
import javax.swing.Timer;

/**
 * The GUI (View) portion of the code
 * This class will draw the elements of the game, when called by the
 * WarGame class (Controller)
 *
 * @author  Allison Jensen
 */
public class WarGUI extends JFrame {
	
	private static final long serialVersionUID = 1;
	
	/**
	 * Reference to thw War model
	 */
	War gameLogic;
	
	/* Global variables to be used for the menu bar*/
	JMenuBar menuBar;
	JMenu menu;
	JMenuItem menuItem1;
	JMenuItem menuItem2;
	JMenuItem menuItem3;
	JMenuItem menuItem4;
	
	/* The panels of the screen */
	JPanel myPanel; //The main panel 
	JPanel gambleComputerPanel;
	JPanel computerDrawPilePanel;
	JPanel computerWonPilePanel;
	JPanel playerDrawPilePanel;
	JPanel playerWonPilePanel;
	JLabel playerMovingPanel;
	JLabel computerMovingPanel;
	
	/* The labelarray of cards for the panels*/
	JLabel [] playerGamblePile;
	JLabel [] computerGamblePile;
	JLabel [] playerWonPile;
	JLabel [] computerWonPile;
	JLabel [] playerDrawPile;
	JLabel [] computerDrawPile;
	
	/* The computer's 5 cards to choose from */
	JLabel [] computerCards;
	JLabel computerCard1;
	JLabel computerCard2;
	JLabel computerCard3;
	JLabel computerCard4;
	JLabel computerCard5;
	
	/* The player's 5 cards to choose from */
	JLabel [] playerCards;
	JLabel playerCard1;
	JLabel playerCard2;
	JLabel playerCard3;
	JLabel playerCard4;
	JLabel playerCard5;
	
	/* Labels */
	JLabel warLabel = new JLabel("WAR !!!");;
	JLabel warInstructions = new JLabel("Select another card");
	
	/* The positions to display the animation at for the player*/
	int startX;						/* The starting X position of the players card */
	int startY;						/* The starting Y position of the players card */
	int endX = 210;					/* The ending X position of the players card */
	int endY = 350;					/* The ending Y position of the players card */
	int xDistance;					/* The total x distance to be moved	*/
	int xMoveLength;				/* The x distance to move each step */
	int yDistance;					/* The total y distance to be moved */
	int yMoveLength;				/* The y distance to move each step */
	int numberOfSteps = 5;			/* The number of steps for the animation */
	int countOfSteps;			 	/* A count for the number of steps taken */
	
	/* The positions to display the animation for the computer*/
	int cStartX;					/* The starting x position for the computer */
	int cStartY;					/* The starting y position for the computer */
	int cEndX = 210;				/* The ending x position for the computer */
	int cEndY = 200;				/* The ending y position for the computer */
	int cXDistance;					/* The total x distance to be moved */
	int cXMoveLength;				/* The x distance to be move each step*/
	int cYDistance;					/* The total y distance to be moved */
	int cYMoveLength;				/* The y distance to move each step */
	int cNumberOfSteps = 5;			/* The number of steps for the animation */
	int cCountOfSteps;				/* A variable to count the steps of the animation */
	
	/* The timer variables */
	Timer timer;   					/* Timer for initial animation */ 
	Timer timer2;					/* Timer for moving cards to the won piles */
	
	/* The player's current selected card, used for the game Logic*/
	Card playerCard;
	
	/* The computer's current selected care, used for the game Logic */
	Card computerCard;
	
	/* The selected JLabel */
	JLabel selectedPlayerCard;
	
	/* The selected computer card */
	JLabel selectedComputerCard;
	
	/* Debugging Variable */
	private boolean debug = false;
	
	/* Variable to keep track if there is a war */
	boolean isWar;

	/* Variables to use in animation of cards to the won piles */
	private int wonStartX;			/* The starting x position for the player */
	private int wonStartY;			/* The starting y position for the player */
	private int cWonStartX;			/* The starting x position for the computer */
	private int cWonStartY;			/* The starting y position for the computer */
	private int wonEndX;			/* The ending x position for the won cards */
	private int wonEndY;			/* The ending y position for the won cards */
	private int yWonMoveLength;		/* The y length to move from the players cards to the won pile */
	private int xWonMoveLength;		/* The x length to move from the players cards to the won pile */
	private int cXWonMoveLength;	/* The x length to move from the computers cards to the won pile */
	private int cYWonMoveLength;	/* The y length to move from the computers cards to the won pile */
	private int wonSteps;			/* The total number of steps for the animation */
	private int wonStepCount;		/* A counter to keep track of steps taken in animation */
	private int animationCounter;	/* An animation counter */
	
	/* Keeps track of the current players for animations */
	private Player computerPlayer;
	private Player humanPlayer;
	
	/* Keeps track if the player is the winner of the round */
	private boolean playerWinner;
	
	/* Keeps track if animation is going on or not */
	private boolean animating;
	
	
	/**
	 * A modifier for the war variable
	 * @param war true if war, false otherwise
	 */
	public void setIsWar(boolean war) {
		isWar = war;
	}
	
	/**
	 * A modifier for setting the player object
	 * @param p the human player
	 */
	public void setHumanPlayer( Player p) {
		humanPlayer = p;
	}
	
	/**
	 * A modifier for setting the computer player object
	 * @param c the computer player
	 */
	public void setComputerPlayer(Player c) {
		computerPlayer = c;
	}
	
	/**
	 * A Modifier to set if the player is the current winner
	 * @param b true if the human player is the winner, false otherwise
	 */
	public void isPlayerWinner(boolean b) {
		playerWinner = b;
	}
	
	/** Creates new form WarGUI */
	public WarGUI() {
		//System.out.prinln("Starting War GUI");
		initComponents();
		gameLogic = new War("Player", this);
	}
	
	
	/**
	 * Initializes all of the components
	 */
	private void initComponents() {
		//System.out.prinln("INITIALIZING COMPONENTS");
		//Setting up the JFrame
		this.setDefaultCloseOperation(EXIT_ON_CLOSE);
		this.setSize(new Dimension(1000, 730));
		
		isWar = false;
		animating = false;
		playerWinner = false;
		
		//Setting up the menus
		menuBar = new JMenuBar();
		
		menu = new JMenu("File");
		menu.setMnemonic(KeyEvent.VK_A);
		menuBar.add(menu);
		
		menuItem1 = new JMenuItem ("New Game" , KeyEvent.VK_T);
		menuItem1.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_N, ActionEvent.CTRL_MASK));
		menu.add(menuItem1);
		menuItem1.addActionListener(new TempListener());
		
		menuItem2 = new JMenuItem ("Quit", KeyEvent.VK_B);
		menuItem2.addActionListener(new TempListener());
		menu.add(menuItem2);
		
		menu = new JMenu("Help");
		menu.setMnemonic(KeyEvent.VK_H);
		menuBar.add(menu);
		
		menuItem3 = new JMenuItem ("How to play");
		menuItem3.addActionListener(new TempListener());
		menu.add(menuItem3);
		
		
		this.setJMenuBar(menuBar);
		
		// Initializing the panels
		myPanel = new JPanel() {
			public void paintComponent(Graphics g) {
				ImageIcon icon = new ImageIcon(getImageURL("images/felt.JPG"));
				g.drawImage(icon.getImage(), 0, 0, null);
				setOpaque( false );
				super.paintComponent(g);
			}
		};
		computerDrawPilePanel = new JPanel(){
			public void paintComponent(Graphics g) {
				ImageIcon icon = new ImageIcon(getImageURL("images/felt.JPG"));
				g.drawImage(icon.getImage(), 0, 0, null);
				setOpaque( false );
				super.paintComponent(g);
			}
		};
		computerWonPilePanel = new JPanel(){
			public void paintComponent(Graphics g) {
				ImageIcon icon = new ImageIcon(getImageURL("images/felt.JPG"));
				g.drawImage(icon.getImage(), 0, 0, null);
				setOpaque( false );
				super.paintComponent(g);
			}
		};
		playerDrawPilePanel = new JPanel(){
			public void paintComponent(Graphics g) {
				ImageIcon icon = new ImageIcon(getImageURL("images/felt.JPG"));
				g.drawImage(icon.getImage(), 0, 0, null);
				setOpaque( false );
				super.paintComponent(g);
			}
		};
		playerWonPilePanel = new JPanel(){
			public void paintComponent(Graphics g) {
				ImageIcon icon = new ImageIcon(getImageURL("images/felt.JPG"));
				g.drawImage(icon.getImage(), 0, 0, null);
				setOpaque( false );
				super.paintComponent(g);
			}
		};
		gambleComputerPanel = new JPanel() {
			public void paintComponent(Graphics g) {
				ImageIcon icon = new ImageIcon(getImageURL("images/felt.JPG"));
				g.drawImage(icon.getImage(), 0, 0, null);			
				setOpaque( false );
				super.paintComponent(g);
			}
		};
		gambleComputerPanel.setBounds(200, 0, 600, 800);
		gambleComputerPanel.setLayout(null);
		computerDrawPilePanel.setBounds(0, 0, 200, 200);
		computerDrawPilePanel.setLayout(null);
		playerDrawPilePanel.setBounds(0, 500, 200, 200);
		playerDrawPilePanel.setLayout(null);
		computerWonPilePanel.setBounds(800, 0, 200, 200);
		computerWonPilePanel.setLayout(null);
		playerWonPilePanel.setBounds(800, 500, 200, 200);
		playerWonPilePanel.setLayout(null);
		
		//Setting up myPanel
		myPanel.setLayout(null);
		myPanel.setPreferredSize(new Dimension(1000,700));
		myPanel.setBackground(new Color(0, 102, 51));
		
		//Adding the components panels to the main myPanel
		myPanel.add(gambleComputerPanel);
		myPanel.add(computerDrawPilePanel);
		myPanel.add(computerWonPilePanel);
		myPanel.add(playerDrawPilePanel);
		myPanel.add(playerWonPilePanel);
		
		
		computerCards = new JLabel[5];
		computerCard1 = new JLabel();
		computerCard2 = new JLabel();
		computerCard3 = new JLabel();
		computerCard4 = new JLabel();
		computerCard5 = new JLabel();
		computerCards[0] = computerCard1;
		computerCards[1] = computerCard2;
		computerCards[2] = computerCard3;
		computerCards[3] = computerCard4;
		computerCards[4] = computerCard5;
		// Intializing player cards and panel 
		playerCards = new JLabel[5];
		playerCard1 = new JLabel();
		playerCard2 = new JLabel();
		playerCard3 = new JLabel();
		playerCard4 = new JLabel();
		playerCard5 = new JLabel();
		playerCards[0] = playerCard1;
		playerCards[1] = playerCard2;
		playerCards[2] = playerCard3;
		playerCards[3] = playerCard4;
		playerCards[4] = playerCard5;
		
		//Putting labels on the gamble computer panel
		JLabel pCards = new JLabel ("Player's Cards");
		gambleComputerPanel.add(pCards);
		pCards.setBounds(200, 510, 200, 50);
		pCards.setFont(new Font("Serif", Font.BOLD, 20));
		pCards.setForeground(Color.WHITE);
		
		JLabel cCards = new JLabel ("Computer's Cards");
		gambleComputerPanel.add(cCards);
		cCards.setBounds(190, 120, 200, 50);
		cCards.setFont(new Font("Serif", Font.BOLD, 20));
		cCards.setForeground(Color.WHITE);
		
		
		
		// Setting up the computer Cards 
		computerCard1.setIcon(new ImageIcon(getImageURL("images/back1.JPG")));
		//computerHandPanel.add(computerCard1);
		gambleComputerPanel.add(computerCard1);
		computerCard1.setBounds(10, 10, 100, 125);
		computerCard1.addMouseListener(new mouseListener());
		computerCard2.setIcon(new ImageIcon(getImageURL("images/back1.JPG")));
		//computerHandPanel.add(computerCard2);
		gambleComputerPanel.add(computerCard2);
		computerCard2.setBounds(120, 10, 100, 125);
		computerCard2.addMouseListener(new mouseListener());
		computerCard3.setIcon(new ImageIcon(getImageURL("images/back1.JPG")));
		//computerHandPanel.add(computerCard3);
		gambleComputerPanel.add(computerCard3);
		computerCard3.setBounds(230, 10, 100, 125);
		computerCard3.addMouseListener(new mouseListener());
		computerCard4.setIcon(new ImageIcon(getImageURL("images/back1.JPG")));
		//computerHandPanel.add(computerCard4);
		gambleComputerPanel.add(computerCard4);
		computerCard4.setBounds(340, 10, 100, 125);
		computerCard4.addMouseListener(new mouseListener());
		computerCard5.setIcon(new ImageIcon(getImageURL("images/back1.JPG")));
		//computerHandPanel.add(computerCard5);
		gambleComputerPanel.add(computerCard5);
		computerCard5.setBounds(450, 10, 100, 125);
		computerCard5.addMouseListener(new mouseListener());
		
		// Setting up the player panel 
		playerCard1.setIcon(new ImageIcon(getImageURL("images/back1.JPG")));
		//playerHandPanel.add(playerCard1);
		gambleComputerPanel.add(playerCard1);
		playerCard1.setBounds(10, 550, 100, 125);
		playerCard1.addMouseListener(new mouseListener());
		playerCard2.setIcon(new ImageIcon(getImageURL("images/back1.JPG")));
		//playerHandPanel.add(playerCard2);
		gambleComputerPanel.add(playerCard2);
		playerCard2.setBounds(120, 550, 100, 125);
		playerCard2.addMouseListener(new mouseListener());
		playerCard3.setIcon(new ImageIcon(getImageURL("images/back1.JPG")));
		//playerHandPanel.add(playerCard3);
		gambleComputerPanel.add(playerCard3);
		playerCard3.setBounds(230, 550, 100, 125);
		playerCard3.addMouseListener(new mouseListener());
		playerCard4.setIcon(new ImageIcon(getImageURL("images/back1.JPG")));
		//playerHandPanel.add(playerCard4);
		gambleComputerPanel.add(playerCard4);
		playerCard4.setBounds(340, 550, 100, 125);
		playerCard4.addMouseListener(new mouseListener());
		playerCard5.setIcon(new ImageIcon(getImageURL("images/back1.JPG")));
		//playerHandPanel.add(playerCard5);
		gambleComputerPanel.add(playerCard5);
		playerCard5.setBounds(450, 550, 100, 125);
		playerCard5.addMouseListener(new mouseListener());
		
		playerMovingPanel = new JLabel();
		playerMovingPanel.setPreferredSize(new Dimension(100, 125));
		computerMovingPanel = new JLabel();
		computerMovingPanel.setPreferredSize(new Dimension(100, 125));
		
		this.getContentPane().add(myPanel);        
		//System.out.prinln("HERE");
		pack();
	}
	
	
	/**
	 * Updates the player's and the computer's main gamble piles.
	 * @param p the human player
	 * @param c the computer player
	 */
	public void updateGamblePiles(Player p, Player c) {
		drawPlayerGamblePile(c.getGamblePile(), p.getGamblePile());
		gambleComputerPanel.repaint();
	}
	
	/**
	 * Updates the draw and won piles for both the computer and the player
	 * 
	 */
	public void updatePiles() {
		drawPlayerPile(humanPlayer.getDrawPile().cardCount());
		drawPlayerWonPile(humanPlayer.getWonPile().cardCount());        
		drawComputerPile(computerPlayer.getDrawPile().cardCount());
		drawComputerWonPile(computerPlayer.getWonPile().cardCount());
		playerDrawPilePanel.repaint();
		playerWonPilePanel.repaint();
		computerDrawPilePanel.repaint();
		computerWonPilePanel.repaint();
	}
	
	/**
	 * Displays a game over window
	 *
	 */
	public void gameOver(Player winner, String message) {
		//JLabel label = new JLabel ("GAME OVER!!!");
		//label.setFont(new Font("Serif", Font.BOLD, 36));
		//label.setForeground(Color.WHITE);
		//label.setBounds(180, 250, 200, 50);
		//myPanel.add(label);
		//myPanel.repaint();
		this.setEnabled(false);
		GameOver gameOver = new GameOver(winner, this, message);
		
	}
	
	/**
	 * Starts a new game and resets all of the WarGUI components
	 */
	public void reset() {
		this.setEnabled(true);
		this.setVisible(false);
		java.awt.EventQueue.invokeLater(new Runnable() {
			public void run() {
				new WarGUI().setVisible(true);
			}
		});
		
	}
	
	/**
	 * Displays the label on the right of the screen when a war is taking place.
	 */
	public void war() {
		warLabel.setFont(new Font("Serif" , Font.BOLD, 36));
		warLabel.setForeground(Color.WHITE);
		warLabel.setBounds(30, 275, 180, 50);
		gambleComputerPanel.add(warLabel);
		warInstructions.setFont(new Font("Serif", Font.BOLD, 20));
		warInstructions.setForeground(Color.WHITE);
		warInstructions.setBounds(10, 325, 180, 50);
		gambleComputerPanel.add(warInstructions);
	}
	
	/**
	 * Returns the URL for the images
	 * @param imageName
	 * @return
	 */
	public static URL getImageURL(String imageName) {
		URL imageURL = null;
		try {
			Class cardClass = Class.forName("WarGUI");
			URL classURL = cardClass.getResource("WarGUI.class");
			imageURL = new URL(classURL, imageName);
		} catch (Exception e) {
			System.out.println(e);
			System.exit(0);
		}
		return imageURL;
	}
	
	/**
	 * Removes the WAR label when a war is not taking place.
	 */
	private void removeWar() {
		if (warLabel != null) {
			gambleComputerPanel.remove(warLabel);
			gambleComputerPanel.remove(warInstructions);
			gambleComputerPanel.repaint();
		}
	}
	
	
	/**
	 * Draws the human player's gamble pile.
	 * 
	 * @param pile the human player's cards in play
	 */
	private void drawPlayerGamblePile(CardPile computerPile, CardPile playerPile) {
		int i = computerPile.cardCount();
		if (playerGamblePile != null && computerGamblePile != null) {
			//removing all components from this panel
			if (playerGamblePile.length != 0 && computerGamblePile.length != 0) {
				Component [] comps = gambleComputerPanel.getComponents();
				for (int j = 0; j < comps.length; j++) {
					//gambleComputerPanel.remove(comps[j]);
				}
				for (int j = 0; j < computerGamblePile.length; j++) {
					gambleComputerPanel.remove(computerGamblePile[j]);
				}
				for (int j = 0; j < playerGamblePile.length; j++) {
					gambleComputerPanel.remove(playerGamblePile[j]);
				}
			}
		}
		computerGamblePile = new JLabel[computerPile.cardCount()];
		playerGamblePile = new JLabel[playerPile.cardCount()];
		int startPosition = (200 + (10  * i));
		for (int j = 0; j < playerGamblePile.length; j++) {
			Card currentCard = playerPile.getCardAt(j);
			if (debug) {
				System.out.println("DRAWING  P CARD: " + playerPile.getCardAt(j).toString() + "At position: " + startPosition);
			}
			playerGamblePile[j] = new JLabel();
			playerGamblePile[j].setBounds(startPosition, 350, 100, 125);
			String image = getImage(currentCard);
			playerGamblePile[j].setIcon(new ImageIcon(getImageURL(image)));
			playerGamblePile[j].setDisabledIcon(new ImageIcon(getImageURL(image)));
			playerGamblePile[j].setEnabled(false);
			//System.out.println(playerGamblePile[j].getBounds());
			gambleComputerPanel.add(playerGamblePile[j]);
			//Going through cards and setting offset based on whether they are
			//face up or face down
			if (j < (playerGamblePile.length - 1) ) {
				if (playerPile.getCardAt(j + 1).getFaceUp()) {
					startPosition -= 30;
				} else {
					startPosition -= 10;
				}
			}else {
				startPosition -= 30;
			}
		}
		// Putting images on the proper cards, setting the war cards to black
		for (int j = 0; j < playerGamblePile.length; j++) {
			if ((j % 4) == 0) {
				//playerGamblePile[j].setBackground(Color.GREEN);
			} else {
				playerGamblePile[j].setBackground(Color.BLACK);
			}
		}
		//Drawing the computer gamble piles
		startPosition = 200 + (10 * i);
		for (int j = 0; j < computerGamblePile.length; j++) {
			Card currentCard = computerPile.getCardAt(j);
			if (debug) {
				System.out.println("DRAWING C CARD: " + computerPile.getCardAt(j).toString() + "At position: " + startPosition);
			}
			computerGamblePile[j] = new JLabel();
			computerGamblePile[j].setBounds(startPosition, 200, 100, 125);
			String image = getImage(currentCard);
			computerGamblePile[j].setIcon(new ImageIcon(getImageURL(image)));
			computerGamblePile[j].setDisabledIcon(new ImageIcon(getImageURL(image)));
			computerGamblePile[j].setEnabled(false);
			//System.out.println(computerGamblePile[j].getBounds());
			gambleComputerPanel.add(computerGamblePile[j]);
			//Going through cards and setting offset based on whether they are
			//face up or face down
			if (j < (computerGamblePile.length - 1) ) {
				if (computerPile.getCardAt(j + 1).getFaceUp()) {
					startPosition -= 30;
				} else {
					startPosition -= 10;
				}
			}else {
				startPosition -= 30;
			}
		}
		// Putting images on the proper cards, setting the war cards to black
		for (int j = 0; j < computerGamblePile.length; j++) {
			if ((j % 4) == 0) {
				//playerGamblePile[j].setBackground(Color.GREEN);
			} else {
				computerGamblePile[j].setBackground(Color.BLACK);
			}
		}
		// System.out.println("P PILE: " + pPile.cardCount());
	}
	
	/**
	 * Draws the computer's gamble pile.
	 * 
	 * @param pile the computer players cards in play
	 */
	private void drawComputerGamblePile(CardPile pile) {
		int i = pile.cardCount();
		if (computerGamblePile != null) {
			//removing all components from this panel 
			if (computerGamblePile.length != 0) {
				Component [] comps = gambleComputerPanel.getComponents();
				for (int j = 0; j < comps.length; j++) {
					gambleComputerPanel.remove(comps[j]);
				}
			}
		}
		computerGamblePile = new JLabel[i];
		int counter = 0;
		int startPosition = (i * 10);
		for (int j = 0; j < computerGamblePile.length; j++) {
			Card currentCard = pile.getCardAt(j);
			if (debug) {
				System.out.println("DRAWING CARD: " + pile.getCardAt(j).toString() + "At position: " + startPosition);
			}
			computerGamblePile[j] = new JLabel();
			computerGamblePile[j].setBounds(startPosition, 0, 100, 125);
			String image = getImage(currentCard);
			computerGamblePile[j].setIcon(new ImageIcon(getImageURL(image)));
			computerGamblePile[j].setDisabledIcon(new ImageIcon(getImageURL(image)));
			computerGamblePile[j].setEnabled(false);
			//System.out.println(computerGamblePile[j].getBounds());
			gambleComputerPanel.add(computerGamblePile[j]);
			//Going through cards and setting offset based on whether they are
			//face up or face down
			if (j < (computerGamblePile.length - 1) ) {
				if (pile.getCardAt(j + 1).getFaceUp()) {
					startPosition -= 30;
				} else {
					startPosition -= 10;
				}
			}else {
				startPosition -= 30;
			}
		}
		// Putting images on the proper cards, setting the war cards to black
		for (int j = 0; j < computerGamblePile.length; j++) {
			if ((j % 4) == 0) {
				//playerGamblePile[j].setBackground(Color.GREEN);
			} else {
				computerGamblePile[j].setBackground(Color.BLACK);
			}
		}
	}
	
	
	/**
	 * Sets up the components to draw the computer's draw pile.
	 * 
	 * @param i the number of cards in the computers draw pile
	 */
	private void drawComputerPile(int i) {
		if (computerDrawPile != null) {
			//System.out.println("computer Draw Pile: " + computerDrawPile.length);
			Component [] comps = computerDrawPilePanel.getComponents();
			for (int j = 0; j < comps.length; j++) {
				computerDrawPilePanel.remove(comps[j]);
			}
		}
		computerDrawPile = new JLabel[i];
		int xPosition = 15 + i;
		int yPosition = 5 + i;
		JLabel label = new JLabel ("DRAW PILE:  " + computerDrawPile.length  
				+ " CARDS");
		label.setFont(new Font("Serif", Font.BOLD, 16));
		label.setForeground(Color.WHITE);
		label.setBounds(10, 180, 200, 20);
		computerDrawPilePanel.add(label);
		
		for (int j = 0; j < computerDrawPile.length; j++) {
			computerDrawPile[j] = new JLabel();
			computerDrawPile[j].setIcon(new ImageIcon(getImageURL("images/back1.JPG")));
			computerDrawPile[j].setBounds(xPosition, yPosition, 100, 125);
			computerDrawPile[j].setDisabledIcon(new ImageIcon(getImageURL("images/back1.JPG")));
			computerDrawPile[j].setEnabled(false);
			computerDrawPilePanel.add(computerDrawPile[j]);
			yPosition -= 1;
			xPosition += 1;
		}
	}
	
	/**
	 * Sets up the components to draw the computer's won pile.
	 * 
	 * @param i the number of cards in the computers won pile
	 */
	private void drawComputerWonPile(int i) {
		if (computerWonPile != null) {
			if (debug) {
				System.out.println("computer Won Pile: " + computerWonPile.length);
			}
			Component [] comps = computerWonPilePanel.getComponents();
			for (int j = 0; j < comps.length; j++) {
				computerWonPilePanel.remove(comps[j]);
			}
		}
		computerWonPile = new JLabel[i];
		int xPosition = 5 + i;
		int yPosition = 5 + i;
		JLabel label = new JLabel ("WON PILE:  " + computerWonPile.length  
				+ " CARDS");
		label.setFont(new Font("Serif", Font.BOLD, 16));
		label.setForeground(Color.WHITE);
		label.setBounds(0, 180, 200, 20);
		computerWonPilePanel.add(label);
		for (int j = 0; j < computerWonPile.length; j++) {
			computerWonPile[j] = new JLabel();
			computerWonPile[j].setIcon(new ImageIcon(getImageURL("images/back1.JPG")));
			computerWonPile[j].setBounds(xPosition, yPosition, 100, 125);
			computerWonPile[j].setDisabledIcon(new ImageIcon(getImageURL("images/back1.JPG")));
			computerWonPile[j].setEnabled(false);
			computerWonPilePanel.add(computerWonPile[j]);
			yPosition -= 1;
			xPosition += 1;
		}
	}
	
	/**
	 * Draws the cards in the player's draw pile.
	 * 
	 * @param i the number of cards in their pile
	 */
	private void drawPlayerPile(int i) {
		if (playerDrawPile != null) {
			if (debug) {
				System.out.println("player Draw Pile: " + playerDrawPile.length);
			}
			Component [] comps = playerDrawPilePanel.getComponents();
			for (int j = 0; j < comps.length; j++) {
				playerDrawPilePanel.remove(comps[j]);
			}
		}
		playerDrawPile = new JLabel[i];
		int xPosition = 15 + i;
		int yPosition = 60;
		JLabel label = new JLabel ("DRAW PILE:  " + playerDrawPile.length  
				+ " CARDS");
		label.setFont(new Font("Serif", Font.BOLD, 16));
		label.setForeground(Color.WHITE);
		label.setBounds(10, 0, 200, 20);
		playerDrawPilePanel.add(label);
		for (int j = 0; j < playerDrawPile.length; j++) {
			playerDrawPile[j] = new JLabel();
			playerDrawPile[j].setIcon(new ImageIcon(getImageURL("images/back1.JPG")));
			playerDrawPile[j].setBounds(xPosition, yPosition, 100, 125);
			playerDrawPile[j].setDisabledIcon(new ImageIcon(getImageURL("images/back1.JPG")));
			playerDrawPile[j].setEnabled(false);
			playerDrawPilePanel.add(playerDrawPile[j]);
			yPosition -= 1;
			xPosition += 1;
		}
	}
	
	
	/**
	 * Draws the cards in the player's won pile.
	 *
	 * @param i the number of cards in the player's won pile
	 */
	private void drawPlayerWonPile(int i) { 
		if (playerWonPile != null) {
			if (debug) {
				System.out.println("player won Pile: " + playerWonPile.length);
			}
			Component [] comps = playerWonPilePanel.getComponents();
			for (int j = 0; j < comps.length; j++) {
				playerWonPilePanel.remove(comps[j]);
			}
		}
		playerWonPile = new JLabel[i];
		int xPosition = 5 + i;
		int yPosition = 60;
		JLabel label = new JLabel ("WON PILE:  " + playerWonPile.length  
				+ " CARDS");
		label.setFont(new Font("Serif", Font.BOLD, 16));
		label.setForeground(Color.WHITE);
		label.setBounds(0, 0, 200, 20);
		playerWonPilePanel.add(label);
		for (int j = 0; j < playerWonPile.length; j++) {
			playerWonPile[j] = new JLabel();
			playerWonPile[j].setIcon(new ImageIcon(getImageURL("images/back1.JPG")));
			playerWonPile[j].setBounds(xPosition, yPosition, 100, 125);
			playerWonPile[j].setDisabledIcon(new ImageIcon(getImageURL("images/back1.JPG")));
			playerWonPile[j].setEnabled(false);
			playerWonPilePanel.add(playerWonPile[j]);
			yPosition -= 1;
			xPosition += 1;
		}
	}
	
	/**
	 * Returns the image that corresponds to the input card.
	 * 
	 * @param c Card to get the image for
	 * @return String representation of the image
	 */
	private String getImage(Card c) {
		if (c == null) {
			//System.out.prinln("CARD IS NULL");
			return "images/back1.JPG";
		} else if(c.getFaceUp()) {
			//System.out.println("CARD IS FACE UP");
			String str = "images/cards/";
			int rank = c.getRank();
			String rankStr = rank + "";
			//System.out.println(rankStr);
			if (rank >= 10) {
				if (rank == 11) {
					rankStr = "j";
				} else if (rank == 12) {
					rankStr = "q";
				} else if (rank == 13) {
					rankStr = "k";
				} else if (rank == 14) {
					rankStr = "a";
				}
			}
			
			String suit = "";
			if (c.getSuit().equals("Hearts")) {
				suit = "h";
			} else if (c.getSuit().equals("Diamonds")) {
				suit = "d";
			} else if (c.getSuit().equals("Spades")) {
				suit = "s";
			} else {
				suit = "c";
			}
			str += rankStr;
			str += suit;
			str += ".gif";
			//System.out.println(str);
			return str;
		}
		else {
			return "images/back1.JPG";
		}
	}
	
	/**
	 * Animates the given cards
	 * 
	 * @param playerCard
	 * @param computerCard
	 */
	private void animateCard(JLabel playerCard, JLabel computerCard) {
		numberOfSteps = 5;
		countOfSteps = 0;
		animationCounter = 0;
		playerMovingPanel.setIcon(playerCard.getIcon());
		computerMovingPanel.setIcon(computerCard.getIcon());
		gambleComputerPanel.add(playerMovingPanel);
		gambleComputerPanel.add(computerMovingPanel);
		startX = playerCard.getX();
		startY = playerCard.getY();
		cStartX = computerCard.getX();
		cStartY = computerCard.getY();
		move(startX, startY, endX, endY, cStartX, cStartY, cEndX, cEndY);
		
	}
	
	/**
	 * Animates the won cards to the won piles
	 */
	public void animateWonCard() { 
		if (playerGamblePile != null && computerGamblePile != null) {
			//removing all components from this panel
			if (playerGamblePile.length != 0 && computerGamblePile.length != 0) {
				//System.out.prinln("REMOVING COMPONENTS");
				Component [] comps = gambleComputerPanel.getComponents();
				for (int j = 0; j < comps.length; j++) {
					//gambleComputerPanel.remove(comps[j]);
				}
				for (int j = 0; j < computerGamblePile.length; j++) {
					gambleComputerPanel.remove(computerGamblePile[j]);
				}
				for (int j = 0; j < playerGamblePile.length; j++) {
					gambleComputerPanel.remove(playerGamblePile[j]);
				}
			}
		}
		if (warLabel != null) {
		    removeWar();
		}
		gambleComputerPanel.repaint();
		wonSteps = 5;
		wonStepCount = 0;
		wonStartX = 300;
		wonStartY = 350;
		cWonStartX = 300;
		cWonStartY = 200;
		if (playerWinner) {
			wonEndX = 700;
			wonEndY = 600;
		} else {
			wonEndX = 700;
			wonEndY = 100;
		}
		moveToWonPile(wonStartX, wonStartY, wonEndX, wonEndY, cWonStartX, cWonStartY);
	}
	
	/**
	 * Calculates the move distances and starts the animations for the won piles
	 * 
	 * @param startX the players starting x position
	 * @param startY the players starting y position
	 * @param endX	the ending x position of the won pile
	 * @param endY  the ending y position of the won pile
	 * @param cStartX the computers starting x position
	 * @param cStartY the computers starting y position
	 */
	private void moveToWonPile(int startX, int startY, int endX, int endY, int cStartX, int cStartY) {
		//System.out.println("In move");
		if (cStartX < endX && cStartY > endY) {
			//move right and up
			//System.out.println("MOVING RIGHT AND up");
			cXDistance = endX - cStartX;
			cYDistance = endY + cStartY;
		} else if (cStartX < endX && cStartY < endY) {
			//move right and down
			//System.out.println("MOVING LEFT AND DOWN");
			cXDistance = endX - cStartX;
			cYDistance = endY - cStartY;
		} 
		//System.out.prinln("X DISTANCE: " + cXDistance);
		//System.out.prinln("Y DISTANCE: " + cYDistance);
		cXWonMoveLength = cXDistance/wonSteps;
		cYWonMoveLength = cYDistance/wonSteps;		
		if (startX < endX && startY > endY) {
			//move right and up
			int xDistance = wonEndX - wonStartX;
			int yDistance = wonStartY - wonEndY;
			xWonMoveLength = xDistance/wonSteps;
			yWonMoveLength = yDistance/wonSteps;  
			timer2 = new Timer(100, timerListener2);
			timer2.start();
			animating = true;
		} else if (startX < endX && startY < endY) {
			//move right and down
			//System.out.prinln("MOVING RIGHT AND DOWN");
			int xDistance = wonEndX - wonStartX;
			int yDistance = wonEndY - wonStartY;
			xWonMoveLength = xDistance/wonSteps;
			yWonMoveLength = yDistance/wonSteps;
			timer2 = new Timer (100, timerListener2);
			timer2.start();
			animating = true;
		}
	}
	
	
	  /**
	   * Calculates the move distances and starts the animation.
	   *
	   * @param startX the players selected card starting X position
	   * @param startY the players selected card starting Y position
	   * @param endX the players ending X position for the card
	   * @param endY the players ending Y position for the card
	   * @param cStartX the computers selected card starting X position
	   * @param cStartY the computers selected card starting Y position
	   * @param cEndX the computers ending X position for the card
	   * @param cEndY the computers ending Y position for the card
	   */

	private void move(int startX, int startY, int endX, int endY, int cStartX, int cStartY, int cEndX, int cEndY) {
		//System.out.println("In move");
		if (cStartX < cEndX && cStartY < cEndY) {
			//move right and down
			//System.out.println("MOVING RIGHT AND DOWN");
			cXDistance = cEndX - cStartX;
			cYDistance = cEndY - cStartY;
		} else if (cStartX > cEndX && cStartY < cEndY) {
			//move left and down
			//System.out.println("MOVING LEFT AND DOWN");
			cXDistance = cStartX - cEndX;
			cYDistance = cEndY - cStartY;
		} 
		cXMoveLength = cXDistance/numberOfSteps;
		cYMoveLength = cYDistance/numberOfSteps;
		if (startX < endX && startY > endY) {
			//move right and up
			if (debug) {
				System.out.println("Moving right and up");
			}
			xDistance = endX - startX;
			yDistance = startY - endY;
			xMoveLength = xDistance/numberOfSteps;
			yMoveLength = yDistance/numberOfSteps;  
			timer = new Timer(50, timerListener);
			timer.start();
			animating = true;
		} else if (startX < endX && startY < endY) {
			//move right and down
			//System.out.prinln("MOVING RIGHT AND DOWN");
			xDistance = endX - startX;
			yDistance = endY - startY;
			xMoveLength = xDistance/numberOfSteps;
			yMoveLength = yDistance/numberOfSteps;
			timer = new Timer (50, timerListener);
			timer.start();
			animating = true;
		} else if (startX < endX && startY == endY) {
			//move right
			xDistance = endX - startX;
			yDistance = 0;
		} else if (startX > endX && startY > endY) {
			//move left and up
			xDistance = startX - endX;
			yDistance = startY - endY;
			xMoveLength = xDistance/numberOfSteps;
			yMoveLength = yDistance/numberOfSteps;
			timer = new Timer(50, timerListener);
			timer.start();
			animating = true;
		} else if (startX > endX && startY == endY) {
			//move left
			xDistance = startX - endX;
			yDistance = 0;
		} else if (startX == endX && startY < endY) {
			//move down
			xDistance = 0;
			yDistance = endY - startY;
		} else if (startX == endX && startY > endY) {
			//move up 
			xDistance = 0; 
			yDistance = startY - endY;
		} else {
			//Don't move at all
			xDistance = 0;
			yDistance = 0;
		}
	}
	
	

	/**
	 * Action Listener for the timer
	 */
	ActionListener timerListener2 = new ActionListener() {
		
		
		public void actionPerformed(ActionEvent evt) {
			if (wonStepCount == wonSteps) {
				//System.out.prinln("Stopping timer");
				timer2.stop();
				animating = false;
				gambleComputerPanel.remove(playerMovingPanel);
				gambleComputerPanel.remove(computerMovingPanel);
				gambleComputerPanel.repaint();
			}
			playerMovingPanel.setBounds(wonStartX, wonStartY, 100, 125);
			computerMovingPanel.setBounds(cWonStartX, cWonStartY, 100, 125);
			gambleComputerPanel.add(playerMovingPanel);
			gambleComputerPanel.add(computerMovingPanel);
			gambleComputerPanel.repaint();
			if (wonStartX < wonEndX && wonStartY > wonEndY) {
				//move right and up
				wonStartX = wonStartX + xWonMoveLength;
				wonStartY = wonStartY - yWonMoveLength;
			}  else if (wonStartX < wonEndX && wonStartY < wonEndY) {
				//move right and down
				wonStartX = wonStartX + xWonMoveLength;
				wonStartY = wonStartY + yWonMoveLength;
			}
			if (cWonStartX < wonEndX && cWonStartY < wonEndY) {
				//System.out.prinln("IN RIGHT AND DOWN !!!!");
				//move right and down
				cWonStartX = cWonStartX + cXWonMoveLength;
				cWonStartY = cWonStartY + cYWonMoveLength;    			
			} else if (cWonStartX < wonEndX && cWonStartY > wonEndY) {
				//System.out.prinln("IN RIGHT AND UP !!!!");
				//move right and up
				cWonStartX = cWonStartX + cXWonMoveLength;
				cWonStartY = cWonStartY - cYWonMoveLength;
			}
			//System.out.prinln("TIMER FIRED");
			//System.out.prinln("StartX: " + wonStartX);
			//System.out.prinln("StartY: " + wonStartY);
			//System.out.prinln("COMPUTER StartX: " + cWonStartX);
			//System.out.prinln("COMPUTER End X: " + wonEndX);
			//System.out.prinln("COMPUTER StartY: " + cWonStartY);
			//System.out.prinln("COMPUTER EndY: " + wonEndY);
			//System.out.prinln("xMOVELENGTH: " + cXWonMoveLength);
			//System.out.prinln("YMOVE LENGTH: " + cYWonMoveLength);
			
			wonStepCount++;
		}
	};
	
	/**
	 * Action listener for the timer
	 */
	ActionListener timerListener = new ActionListener() {
		// Define an action listener to respond to events
		// from the timer.  When an event is received, the
		// color of the display is changed.
		public void actionPerformed(ActionEvent evt) {
			//System.out.println("COUNT OF STEPS: " + countOfSteps);
			//System.out.println("NUMBER OF STEPS: " + numberOfSteps);
			if (countOfSteps == numberOfSteps) {
				//System.out.prinln("STOPPING TIMER");
				timer.stop();
				animating = false;
				gambleComputerPanel.remove(playerMovingPanel);
				gambleComputerPanel.remove(computerMovingPanel);
				gambleComputerPanel.repaint();
				selectedPlayerCard.setIcon(new ImageIcon(getImageURL("images/back1.JPG")));
				selectedComputerCard.setIcon(new ImageIcon(getImageURL("images/back1.JPG")));
				gameLogic.cardBattle(playerCard, computerCard);
				if (isWar) {
					animationCounter = 0;
					countOfSteps = 0;
				} else {
					//System.out.prinln("Starting Timer");
					timer.start();
					animating = true;
					countOfSteps = 0;
				}
			} else if ((animationCounter == 50 || animationCounter > numberOfSteps) && !isWar) {
				//System.out.println("AnimationCounter Greater");
				if (animationCounter == 50) {
					//System.out.println("AnimationCounter = 10");
					for (int i = 0; i < humanPlayer.getGamblePile().cardCount(); i++) {
						humanPlayer.getGamblePile().getCardAt(i).setFaceUp(true);
						computerPlayer.getGamblePile().getCardAt(i).setFaceUp(true);
					}
					updateGamblePiles(humanPlayer, computerPlayer);
					timer.stop();
					animating = false;
					animationCounter = 0;
					animateWonCard();
					updatePiles();
				} else {
					animationCounter++;
				}
			} else {
				if (debug) {
					System.out.println("TIMER FIRED");
					System.out.println("StartX: " + startX);
					System.out.println("StartY: " + startY);
					
					System.out.println("cStartX: " + cStartX);
					System.out.println("cStartY: " + cStartY);
				}
				playerMovingPanel.setBounds(startX, startY, 100, 125);
				computerMovingPanel.setBounds(cStartX, cStartY, 100, 125);
				gambleComputerPanel.add(playerMovingPanel);
				gambleComputerPanel.add(computerMovingPanel);
				gambleComputerPanel.repaint();
				if (startX < endX && startY > endY) {
					//move right and up
					startX = startX + xMoveLength;
					startY = startY - yMoveLength;
				} else if (startX > endX && startY > endY) {
					//move left and up
					startX = startX - xMoveLength;
					startY = startY - yMoveLength;
				} else if (startX > endX && startY < endY) {
					//move left and down
					//System.out.println("Moving Left and down");
					startX = startX - xMoveLength;
					startY = startY + yMoveLength;
				} else if (startX < endX && startY < endY) {
					//move right and down
					startX = startX + xMoveLength;
					startY = startY + yMoveLength;
				}
				//System.out.println("cEndX: " + cEndX);
				//System.out.println("cEndY: " + cEndY);
				//System.out.println("CX MOVE LENGTH :" + cXMoveLength);
				//System.out.println("CY Move LENGTH :" + cYMoveLength);
				if (cStartX < cEndX && cStartY < cEndY) {
					//move right and down
					cStartX = cStartX + cXMoveLength;
					cStartY = cStartY + cYMoveLength;    			
				} else if (cStartX > cEndX && cStartY < cEndY) {
					//move left and down
					cStartX = cStartX - cXMoveLength;
					cStartY = cStartY + cYMoveLength;
					
				}
				countOfSteps++;
				animationCounter++;
			}
			
		}
	}; 
	
	/**
	 * Returns the computers label for the selected computer card
	 * 
	 * @param index the index of the chosen card
	 * @return the JLabel
	 */
	private JLabel getComputerCardLabel (int index) {
		JLabel retVal = null;
		if (index == 1) {
			retVal =  computerCard1;
		} else if (index == 2) {
			retVal = computerCard2;
		} else if (index == 3) {
			retVal = computerCard3;
		} else if (index == 4) {
			retVal = computerCard4;
		} else if (index == 5) {
			retVal = computerCard5;
		}
		return retVal;
	}
	
	/**
	 * Class for the mouse listeners
	 * @author Allison Jensen
	 *
	 */
	private class mouseListener implements MouseListener {
		public void mouseClicked(MouseEvent e) {
			if (animating) {
				
			} else {
				if (e.getSource().equals(computerCard1)) {              
				}
				if (e.getSource().equals(computerCard2)) {
				}
				if (e.getSource().equals(computerCard3)) {
				}
				if (e.getSource().equals(computerCard4)) {
				}
				if (e.getSource().equals(computerCard5)) {
				}
				if (e.getSource().equals(playerCard1)) {
					playerCard = gameLogic.selectPlayerCard(0);
					playerCard1.setIcon(new ImageIcon(getImageURL(getImage(playerCard))));
					selectedPlayerCard = playerCard1;
					computerCard = gameLogic.getComputerCardSelection();
					int index = gameLogic.getComputerCardSelectedIndex();
					selectedComputerCard = getComputerCardLabel(index + 1);
					selectedComputerCard.setIcon(new ImageIcon(getImageURL(getImage(computerCard))));
					animateCard(playerCard1, selectedComputerCard);
				}
				if (e.getSource().equals(playerCard2)) {
					playerCard = gameLogic.selectPlayerCard(1);
					playerCard2.setIcon(new ImageIcon(getImageURL(getImage(playerCard))));
					selectedPlayerCard = playerCard2;
					computerCard = gameLogic.getComputerCardSelection();
					int index = gameLogic.getComputerCardSelectedIndex();
					selectedComputerCard = getComputerCardLabel(index + 1);
					selectedComputerCard.setIcon(new ImageIcon(getImageURL(getImage(computerCard))));
					animateCard(playerCard2, selectedComputerCard);
				}
				if (e.getSource().equals(playerCard3)) {
					playerCard = gameLogic.selectPlayerCard(2);
					playerCard3.setIcon(new ImageIcon(getImageURL(getImage(playerCard))));
					selectedPlayerCard = playerCard3;
					computerCard = gameLogic.getComputerCardSelection();
					int index = gameLogic.getComputerCardSelectedIndex();
					selectedComputerCard = getComputerCardLabel(index + 1) ;
					selectedComputerCard.setIcon(new ImageIcon(getImageURL(getImage(computerCard))));
					animateCard(playerCard3, selectedComputerCard);
				}
				if (e.getSource().equals(playerCard4)) {
					playerCard = gameLogic.selectPlayerCard(3);
					playerCard4.setIcon(new ImageIcon(getImageURL(getImage(playerCard))));
					selectedPlayerCard = playerCard4;
					computerCard = gameLogic.getComputerCardSelection();
					int index = gameLogic.getComputerCardSelectedIndex();
					selectedComputerCard = getComputerCardLabel(index + 1) ;
					selectedComputerCard.setIcon(new ImageIcon(getImageURL(getImage(computerCard))));
					animateCard(playerCard4, selectedComputerCard);
				}
				if (e.getSource().equals(playerCard5)) {
					
					playerCard = gameLogic.selectPlayerCard(4);
					playerCard5.setIcon(new ImageIcon(getImageURL(getImage(playerCard))));
					selectedPlayerCard = playerCard5;
					computerCard = gameLogic.getComputerCardSelection();
					int index = gameLogic.getComputerCardSelectedIndex();
					selectedComputerCard = getComputerCardLabel(index + 1) ;
					selectedComputerCard.setIcon(new ImageIcon(getImageURL(getImage(computerCard))));
					animateCard(playerCard5, selectedComputerCard);
				}
			}
		}
		
		public void mouseExited (MouseEvent e) {
			
		}
		
		public void mouseReleased (MouseEvent e) {
			
		}
		
		public void mouseEntered (MouseEvent e) {
			
		}
		
		public void mousePressed (MouseEvent e) {
			
		}
	}
	 
	/**
	 * Class to handle menu item actions
	 * @author Allison Jensen
	 *
	 */
	private class TempListener implements ActionListener {
		
		/** Creates an instance of the TempListener */
		public TempListener() {
		} //end of TempListener()
		
		
		/**
		 * A method to implement what to do when a listener is activated
		 * @param e the action event that set off the listener
		 */
		public void actionPerformed(ActionEvent e) {
			if (e.getSource().equals(menuItem1)) {
				reset();           	
			} else if (e.getSource().equals(menuItem2)) {
				System.exit(0);
			} else if (e.getSource().equals(menuItem3)) {
			    //how to play
			    HowToPlay h = new HowToPlay();
				
			} 
		}
	}//end of class tempListener
	
	
	/**
	 * Main Method
	 * @param args the command line arguments
	 */
	public static void main(String args[]) {
		java.awt.EventQueue.invokeLater(new Runnable() {
			public void run() {
				new WarGUI().setVisible(true);
			}
		});
	}
	
	
}
