import java.util.*;
import java.io.*;
import java.net.*;

/**
 * This class simulates a See N Say which has
 * items that can make a noise
 * @author Barb Ericson
 * Copyright 2006
 */
public class SeeNSay
{
  ////////////// fields ////////////////////
  
  /** a list of items that can make a noise */
  private List<NoisyItem> noisyItems = 
    new ArrayList<NoisyItem>();
  
  /** random number generator */
  private Random randNumGen = new Random();
  
  /////////////// constructors /////////////
  
  /** 
   * No-arg constructor 
   */
  public SeeNSay() 
  {
    this.add(new Cat());
    this.add(new Dog());
    this.add(new Horse());
    this.add(new Pig());
  }
  
  /**
   * Constructor that takes a file name
   * @param fileName the name of a file to read from
   */
  public SeeNSay(String fileName)
  {
    loadItemsFromFile(fileName);
  }
  
  ///////////// methods ////////////////////
  
  /**
   * Method to load the items from a file
   * @param fileName the name of the file
   */
  private void loadItemsFromFile(String fileName)
  {
    Class myClass = this.getClass();
    URL fileURL = myClass.getResource(fileName);
    Class itemClass = null;
    String className = null;
    try{
      
    BufferedReader reader = new BufferedReader(
      new FileReader(fileURL.getPath()));
    String line = null;
    while ((line = reader.readLine()) != null)
    {
      className = line.trim();
      itemClass = Class.forName(className);
      noisyItems.add((NoisyItem) itemClass.newInstance());
    }
    } catch (Exception ex) {
      System.out.println("Error in reading from file");
    }
  }
  
  /**
   * Method to get a list of all of the names
   * in the see n say
   * @return a list of the class names for objects
   * in the see n say
   */
  public List<String> getNameList()
  {
    List<String> nameList = 
      new ArrayList<String>();
    Class itemClass = null;
    for (NoisyItem item : noisyItems)
    {
      itemClass = item.getClass();
      nameList.add(itemClass.getName());
    }
    return nameList; 
  }
    
  /**
   * Method to add a noisy item 
   * @param item the item to add
   */
  public void add(NoisyItem item)
  {
    noisyItems.add(item);
  }
  
  /**
   * Method to remove a noisy item
   * @param item the item to remove
   * @return true if success else false
   */
  public boolean remove(NoisyItem item)
  {
    return noisyItems.remove(item);
  }
  
  /**
   * Method to randomly pick an item in the list 
   * @return the picked item
   */
  public NoisyItem pickItem()
  {
    NoisyItem item = null;
    if (noisyItems.size() >= 1)
    {
      int index = randNumGen.nextInt(noisyItems.size());
      item = noisyItems.get(index);
      item.makeNoise();
    }
    return item;
  }
  
  /**
   * Method to pick the item of the passed class
   * with the passed name
   * @param itemName the class name for the item
   * to pick
   */
  public void pickItem(String itemName)
  {
    Class itemClass = null;
    String className = null;
    
    /** Find the item with the passed
     * class name 
     */
    for (NoisyItem item : noisyItems)
    {
      itemClass = item.getClass();
      className = itemClass.getName();
      if (className.equals(itemName))
      {
        item.makeNoise();
        return;
      }
    }
  }
  
  public static void main(String[] args)
  {
    SeeNSay game = new SeeNSay();
    game.pickItem();
  }
  
}