import jm.music.data.*;
import jm.JMC;
import jm.util.*;
import jm.music.tools.*;

public class SongNode extends ListNode
{
  ////////////// fields ////////////////
  
  /////////////// constructors ////////
  
  public SongNode()
  {
    super(new Phrase(),null);
  }
  
  public SongNode(Phrase thePhrase)
  {
    super(thePhrase,null);
  }
  
  public SongNode(Phrase thePhrase, 
                  SongNode theNext)
  {
    super(thePhrase,theNext);
  }
  
  public Phrase getPhrase()
  {
    return (Phrase) super.getValue();
  }
  
  public void setPhrase(Phrase thePhrase)
  {
    super.setValue(thePhrase);
  }
  
  public SongNode getNext()
  {
    return (SongNode) super.getNext();
  }
  
  public void setNext(SongNode theNode)
  {
    super.setNext(theNode);
  }
  
  /**
   * copyNode returns a copy of this node
   * @return another song node with the same notes
   */
  public SongNode copyNode(){
    SongNode copy = new SongNode(this.getPhrase());
    return copy;
  }
  
  /**
   * Accessor for the notes inside the node's phrase
   * @return array of notes and durations inside the phrase
   */
  private Note [] getNotes(){
    return this.getPhrase().getNoteArray();
  }
  
   /**
   * Collect all the notes from this node on
   * in an part (then a score) and open it up for viewing.
   * @param instrument MIDI instrument (program) to be used in playing this list
   */
  public void showFromMeOn(int instrument)
  {
    // Make the Score that we'll assemble the elements into
    // We'll set it up with a default time signature and tempo we like
    // (Should probably make it possible to change these -- maybe with inputs?)
     Score myScore = new Score("My Song");
     myScore.setTimeSignature(3,4);
     myScore.setTempo(120.0);
     
     // Make the Part that we'll assemble things into
     Part myPart = new Part(instrument);
     
     // Make a new Phrase that will contain the notes from all the phrases
     Phrase collector = new Phrase();
     
     // Start from this element (this)
     SongNode current = this;
     
     // While we're not through...
     while (current != null)
     {
       collector.addNoteList(current.getNotes());
       
       // Now, move on to the next element
       current = current.getNext();
     };
     
     // Now, construct the part and the score.
     myPart.addPhrase(collector);
     myScore.addPart(myPart);
     
     // At the end, let's see it!
     View.notate(myScore);

  }
  
  public static void main(String[] args)
  {
    SongNode node1 = new SongNode();
    node1.setPhrase(PhraseLibrary.riff1());
    SongNode node2 = new SongNode();
    node2.setPhrase(PhraseLibrary.riff2());
    SongNode node3 = new SongNode();
    node3.setPhrase(PhraseLibrary.riff3());
    node1.setNext(node2);
    node2.setNext(node3);
    node1.showFromMeOn(JMC.SAX);
  }
  
    
}