
/**
 * Class that represents a student.  A student has a unique id, 
 * first name, last name, nickname, a file with their picture 
 * in it, and an array of grades
 */
public class Student 
{
  
  ////////////// fields //////////////////////////////
  
  /** unique identifier for this student */
  private int id;
  
  /** first name of this student */
  private String firstName;
  
  /** last name of this student */
  private String lastName;
  
  /** nickname for this student */
  private String nickname;
  
  /** the name of a file that has a picture of the student */
  private String pictureFile;
  
  /** An array of grades for this student */
  private double[] grades; 
  
  /** The number of grades set in the grade array */
  private int numGrades = 0;
  
  /** The count of the number of students */
  private static int numStudents = 0; // start with no students
  
  //////////////// constructors //////////////////////////
  
  /** 
   * No argument constructor.  All fields have their default values 
   */
  public Student() 
  {
    // increment the number of students
    numStudents++;
    
    // set the student id to the current count of students
    id = numStudents;
  }
    
  
  /**
   * Constructor that takes the first name, last name, and the number of grades
   * @param firstName the first name to use
   * @param lastName the last name to use
   * @param numGrades the number of grades this student will have
   */
  public Student(String firstName, String lastName, int numGrades)
  {
    
    // use the no-argument constructor to set the id
    this();
    
    // set the first and last name fields to the passed strings
    this.firstName = firstName;
    this.lastName = lastName;
    
    // create the array of grades
    grades = new double[numGrades];
  }
  
  /**
   * Constructor that takes all the data
   * @param firstName the first name to use
   * @param lastName the last name to use
   * @param nickname the nickname to use
   * @param pictureFile the name of the picture file
   * @param numGrades the number of grades 
   */
  public Student(String firstName, String lastName, String nickname, 
                 String pictureFile, int numGrades)
  {
    // use other constructor to handle part of this
    this(firstName, lastName, numGrades);
    
    // set the nickname and picture file
    this.nickname = nickname;
    this.pictureFile = pictureFile;
  }
  
  ////////////////////////// methods //////////////////////////////
  
  /** 
   * Method to get the first name
   * @return the first name
   */
  public String getFirstName() { return firstName; }
  
  /**
   * Method to get the last name
   * @return the last name
   */
  public String getLastName() { return lastName; }
  
  /**
   * Method to get the nickname
   * @return the nickname for this student
   */
  public String getNickname() { return nickname; }
  
  /**
   * Method to get the picture file name 
   * @return the name of the file with a picture of the student in it
   */
  public String getPictureFile() { return pictureFile; }
  
  /**
   * Method to set the first name
   * @param firstName the first name to use
   */
  public void setFirstName(String firstName) { this.firstName = firstName; }
  
  /**
   * Method to set the last name
   * @param lastName the last name to use
   */
  public void setLastName(String lastName) { this.lastName = lastName; }
  
  /**
   * Method to set the nickname
   * @param nickname the nickname to use
   */
  public void setNickname(String nickname) { this.nickname = nickname; }
  
  /**
   * Method to set the picture file name
   * @param fileName the file name of the picture of this student
   */
  public void setPictureName(String fileName) { pictureFile = fileName; }
  
  /**
   * Method to add a grade to the grade array
   * @param grade the grade to add
   */
  public void addGrade(double grade)
  {
    if (grades != null && numGrades < grades.length)
    {
      grades[numGrades] = grade;
      numGrades++;
    }
  }
  
  /** 
   * Method to calculate the grade point 
   * @return the grade point average
   */
  public double getGradePointAverage() 
  {
    double total = 0;
    for (int i = 0; i < numGrades ; i++)
      total = total + grades[i];
    double average = 0;
    
    // if the number of grades is > 0 calcuate the average
    if (numGrades > 0)
      average = total / numGrades;
    
    return average;
  }
  
  /**
   * Method to return a string with information about this student
   */
  public String toString()
  {
    return "Student: " + id + " " + firstName + " " + lastName + " gpa: " + 
      getGradePointAverage();
  }
  
  /**
   * Main method for testing
   */
  public static void main(String[] argv) 
  {
    Student sally = new Student("Sally","Summers",5);
    sally.addGrade(75);
    sally.addGrade(85);
    sally.addGrade(63);
    sally.addGrade(90);
    sally.addGrade(78);
    System.out.println(sally);
  }
    
      
  
  
}