/**
 * Class that describes a customer of a bank
 */
public class Customer
{
  
  /** id for the customer */
  private int id;
  
  /** first name of the customer */
  private String firstName;
  
  /** last name of the customer */
  private String lastName;
  
  /** class variable to hold the next available customer id */
  private static int nextId = 0;
  
  /** account for this customer */
  BankAccount account;
  
  /** 
   * Constructor to create a new customer and create an account for this customer 
   * @param firstName the first name for the customer
   * @param lastName the last name for the customer
   * @param amount the amount to deposit in the new account
   */
  public Customer(String firstName, String lastName, double amount)
  {
    id = ++nextId;
    this.firstName = firstName;
    this.lastName = lastName;
    account = new BankAccount(this,amount);
  }
  
  /**
   * Method to get the id for this customer 
   * @return the id for this customer 
   */
  public int getId() { return id; }
  
  /**
   * Method to check if two customer objects are equal (have the same id)
   * @return true if the id's are the same, else false
   */
  public boolean equals(Object o)
  {
    boolean result = false;
    Customer otherCustomer = null;
    
    if (o instanceof Customer)
    {
      otherCustomer = (Customer) o;
      if (id == otherCustomer.getId())
        result = true;
    }
    return result;
  }
  
  /**
   * Method to get the customer name
   * @return the first and last name in a string
   */
  public String getName() { return firstName + " " + lastName; }
  
  /**
   * Method to get the account 
   * @return the bank account for this customer
   */
  public BankAccount getAccount() { return account; }
  
  /**
   * Method to test 
   */
  public static void main(String[] argv)
  {
    // create a customer and a pirate customer
    Customer customer = new Customer("Barbara", "Ericson", 300);
    Customer pirateCustomer = new Customer("Edward", "Teach", 0);
    
    // get the customer's bank account and the pirate bank account
    BankAccount account = customer.getAccount();
    BankAccount pirateAccount = pirateCustomer.getAccount();
    
    // show the result
    System.out.println(account);
    System.out.println(pirateAccount);
    
    // change the account balance in the customer account
    double amount = account.withdraw(pirateCustomer,200);
    pirateAccount.deposit(amount);
    
    // show the result
    System.out.println(account);
    System.out.println(pirateAccount);
    
  }
}
